


// SPI-Library
#include <SPI.h>
// i2c-Library
#include <Wire.h>
// Library to allow getting PRINT-Statements from Flash instead of RAM
// #include <pgmspace.h>




// Display-Libraries
#include <Adafruit_GFX.h>
#include <gfxfont.h>
#include <Fonts/FreeMono12pt7b.h>
#include <Fonts/FreeMono18pt7b.h>
#include <Fonts/FreeMonoBold24pt7b.h>
#include <Adafruit_HX8357.h>
// Touch-Library
#include <Adafruit_STMPE610.h>


// Setup-Data for TFT-Display
#define TFT_DC 0  // Connect TFT_DC to D3
#define TFT_CS 2  // Connect TFT_CS to D4
Adafruit_HX8357 tft = Adafruit_HX8357(TFT_CS, TFT_DC); // Tell the compiler and Library where the DC and CS-Pins for the TFT are connected
Adafruit_STMPE610 touch = Adafruit_STMPE610(); // Tell the compiler how the STMPE610 Touch Interface is being connected (connected via i2c)

// Assign HEX-Values to color-names
#define BLACK 0x0000
#define BLUE 0x001F
#define RED 0xF800
#define GREEN 0x07E0
#define CYAN 0x07FF
#define MAGENTA 0xF81F
#define YELLOW 0xFFE0
#define WHITE 0xFFFF

// i2c GPIO-Expander Addresses - ATTENTION: Addresses only valid for PCF8574! PCF8574A requires different addresses!
#define ATTEN_ADR 0x20  // i2c-Address of GPIO-Expander that controls the Attenuator
#define KEYBD_ADR 0x21  // i2c Address of GPIO-Expander connected to the Keyboard
#define INTGPIO_ADR 0x22  // i2c Address of the Interrupt-Receiver/GPIO-Expander - INT/GPIO-BITMAP: INT0 | INT 1 | FREE | ADF4351_LOCK | AD9910_LOCK | ADF4351_CS | AD9910_CS | AD9910_DU

#define INT 16

// *** Variables to configure frequency ***
unsigned long Freq = 400000000;     // Frequency-Variable in Hz - Default Value at Startup: 500MHz (ATTENTION: The ADF4351 methods require this value to be expressed in kHz!)
unsigned long Freq_Old = 400000000; // Previous selected frequency in Hz - Default Value at Startup: 100MHz

// *** Variables to configure Signal Amplitude ***
int Atten = 95;             // Attenuation-Variable - Default: 95dB Attenuation
int OUTPUT_dBm = 0;         // Output-Power - 0 = Minimal output-amplitude. Can be adjusted in steps of 0.1 dBm 

// *** Variables for various tasks ***
byte interruptSource = 0; // Stores the Interrupt-Source
bool debugMode = false; // Enables/Disables Debug-Mode
byte intgpio = B11000000; // Stores the current state of the IO-Pins of the INTGPIO-Expander


void setup() {
  Serial.begin(9600);
  Serial.println(F("*** SERIAL INTERFACE - RF SIGNAL-GENERATOR ***"));
  
  Serial.println(F("Defining Pins"));
  pinMode(TFT_DC, OUTPUT);
  pinMode(TFT_CS, OUTPUT);
  
  Serial.println(F("SPI Init"));
  SPI.begin();
  
  Serial.println(F("i2c Init"));
  Wire.begin();
  
  Serial.println(F("DISPLAY-Setup"));
  tft.begin(HX8357D);
  tft.fillScreen(HX8357_BLUE);
  tft.setRotation(1);
  delay(50);
  Serial.println(F("DISPLAY-TEST"));
  TestDisplay();
  Serial.println(F("Show Splashscreen"));
  Splash();
  Serial.println(F("Touch Init"));
  if (! touch.begin()) {
    Serial.println(F("STMPE not found!"));
    ErrorMode(0x10, true);
  }
  
  Serial.println(F("Setup ADF9910 in TEST-MODE"));
  SetupADF9910(0xFF);
  SignalPathTest(false);
  SetupADF9910(0x20);
  
  Serial.println(F("Setup ADF4351 in TEST-MODE"));
  SetupADF4351(0x40);
  SignalPathTest(true);
  SetupADF4351(0x20);

  Serial.println(F("Attaching Interrupt"));
  attachInterrupt(digitalPinToInterrupt(INT), ISR, FALLING);
}

void loop() {
  detachInterrupt(digitalPinToInterrupt(INT));
  SetFreq(600000000);
  delay(1000);
  SetFreq(50000000);
  delay(1000);
  attachInterrupt(digitalPinToInterrupt(INT), ISR, FALLING);
}

/*
   Checks if the Frequency is below or above 400MHz and then calls the Subroutines that program the signal-sources accordingly
*/
void SetFreq(long Freq) {
  if (Freq >= 400000000) {
    SetFreqADF4351(Freq);
  }
  else {
    SetFreqAD9910(Freq);
  }
}


